@@ -37,7 +37,7 @@ def _as_list(value) -> List:
 def _data_period(index) -> Union[pd.Timedelta, Number]:
     """Return data index period as pd.Timedelta"""
     values = pd.Series(index[-100:])
-    return values.diff().median()
+    return values.diff().dropna().median()
 
 
 class _Array(np.ndarray):
@@ -99,9 +99,9 @@ class _Data:
     def __init__(self, df: pd.DataFrame):
         self.__df = df
         self.__i = len(df)
-        self.__pip = None   # type: Optional[float]
-        self.__cache = {}   # type: Dict[str, _Array]
-        self.__arrays = {}  # type: Dict[str, _Array]
+        self.__pip: Optional[float] = None
+        self.__cache: Dict[str, _Array] = {}
+        self.__arrays: Dict[str, _Array] = {}
         self._update()
 
     def __getitem__(self, item):
@@ -111,7 +111,7 @@ class _Data:
         try:
             return self.__get_array(item)
         except KeyError:
-            raise AttributeError("Column '{}' not in data".format(item)) from None
+            raise AttributeError(f"Column '{item}' not in data") from None
 
     def _set_length(self, i):
         self.__i = i
@@ -125,9 +125,9 @@ class _Data:
 
     def __repr__(self):
         i = min(self.__i, len(self.__df) - 1)
-        return '<Data i={} ({}) {}>'.format(i, self.__arrays['__index'][i],
-                                            ', '.join('{}={}'.format(k, v)
-                                                      for k, v in self.__df.iloc[i].items()))
+        index = self.__arrays['__index'][i]
+        items = ', '.join(f'{k}={v}' for k, v in self.__df.iloc[i].items())
+        return f'<Data i={i} ({index}) {items}>'
 
     def __len__(self):
         return self.__i
